@@ -399,9 +399,8 @@ class DockerfileBuilder:
 
 
 def generate_dockerignore(
-    pkg_root: Path, *, wf_type: WorkflowType, overwrite: bool = False
+    dest: Path, *, wf_type: WorkflowType, overwrite: bool = False
 ) -> None:
-    dest = Path(pkg_root) / ".dockerignore"
     if dest.exists():
         if dest.is_dir():
             click.secho(
@@ -415,7 +414,7 @@ def generate_dockerignore(
         ):
             return
 
-    files = [".git", ".github"]
+    files = [".git", ".github", ".venv"]
 
     if wf_type == WorkflowType.nextflow:
         files.extend([".nextflow*", ".nextflow.log*", "work/", "results/"])
@@ -425,11 +424,13 @@ def generate_dockerignore(
     click.secho(f"Successfully generated .dockerignore `{dest}`", fg="green")
 
 
-def get_default_dockerfile(pkg_root: Path, *, wf_type: WorkflowType, overwrite: bool = False):
+def get_default_dockerfile(
+    pkg_root: Path, *, wf_type: WorkflowType, overwrite: bool = False
+):
     default_dockerfile = pkg_root / "Dockerfile"
 
     config = get_or_create_workflow_config(
-        pkg_root=pkg_root,
+        pkg_root / ".latch" / "config",
         base_image_type=BaseImageOptions.nextflow
         if wf_type == WorkflowType.nextflow
         else BaseImageOptions.default,
