@@ -1,3 +1,4 @@
+# ruff: noqa: FBT001, FBT002
 """Entrypoints to service functions through a latch_cli."""
 
 import os
@@ -5,7 +6,7 @@ import sys
 import traceback
 from pathlib import Path
 from textwrap import dedent
-from typing import Callable, List, Optional, Tuple, TypeVar, Union
+from typing import Callable, Optional, TypeVar, Union
 
 import click
 import gql
@@ -13,7 +14,7 @@ from packaging.version import parse as parse_version
 from typing_extensions import ParamSpec
 
 import latch_cli.click_utils
-from latch.ldata._transfer.progress import Progress as _Progress
+from latch.ldata._transfer.progress import Progress as _Progress  # noqa: PLC2701
 from latch.utils import current_workspace
 from latch_cli.click_utils import EnumChoice
 from latch_cli.exceptions.handler import CrashHandler
@@ -68,10 +69,8 @@ def requires_login(f: Callable[P, T]) -> Callable[P, T]:
 @click.group("latch", context_settings={"max_content_width": 160})
 @click.version_option(package_name="latch")
 def main():
-    """
-    Collection of command line tools for using the Latch SDK and
-    interacting with the Latch platform.
-    """
+    """Collection of command line tools for using the Latch SDK and interacting with the Latch platform."""
+
     if os.environ.get("LATCH_SKIP_VERSION_CHECK") is not None:
         return
 
@@ -240,6 +239,21 @@ def init(
     type=click.Path(exists=True, dir_okay=False, path_type=Path),
     help="Path to a direnv file (.env) containing environment variables to inject into the container.",
 )
+@click.option(
+    "--output",
+    "-o",
+    type=click.Path(path_type=Path),
+    help="Where to write the result Dockerfile. Default is Dockerfile in the root of the workflow directory.",
+)
+@click.option(
+    "--config-path",
+    type=click.Path(path_type=Path),
+    help=(
+        "Where to read the config to use for generating the Dockerfile. If a config is not found either at"
+        " `config_path` or `config_path / .latch / config`, one will be generated at "
+        "`config_path / .latch / config`. If not provided, it will default to the parent of the output Dockerfile"
+    ),
+)
 def dockerfile(
     pkg_root: Path,
     snakemake: bool = False,
@@ -251,8 +265,10 @@ def dockerfile(
     pyproject: Optional[Path] = None,
     pip_requirements: Optional[Path] = None,
     direnv: Optional[Path] = None,
+    output: Optional[Path] = None,
+    config_path: Optional[Path] = None,
 ):
-    """Generates a user editable dockerfile for a workflow and saves under `pkg_root/Dockerfile`.
+    """Generates a user editable dockerfile for a workflow.
 
     Visit docs.latch.bio to learn more.
     """
@@ -276,10 +292,32 @@ def dockerfile(
         workflow_type = WorkflowType.nextflow
         base_image = BaseImageOptions.nextflow
 
-    config = get_or_create_workflow_config(
-        pkg_root=pkg_root, base_image_type=base_image
+    if output is None:
+        output = pkg_root / "Dockerfile"
+    if output.name != "Dockerfile":
+        output /= "Dockerfile"
+
+    ignore_path = output.with_name(".dockerignore")
+
+    if config_path is None:
+        config_path = output.parent / ".latch" / "config"
+    if config_path.name != "config":
+        config_path /= ".latch/config"
+
+    click.secho(
+        dedent(f"""\
+    The following files will be generated:
+    {click.style("Dockerfile:", fg="bright_blue")} {output}
+    {click.style("Ignore File:", fg="bright_blue")} {ignore_path}
+    {click.style("Latch Config:", fg="bright_blue")} {config_path}
+    """)
     )
 
+    output.parent.mkdir(exist_ok=True, parents=True)
+
+    # todo(ayush): make overwriting this easier
+    config = get_or_create_workflow_config(config_path, base_image_type=base_image)
+
     builder = DockerfileBuilder(
         pkg_root,
         wf_type=workflow_type,
@@ -291,12 +329,9 @@ def dockerfile(
         pip_requirements=pip_requirements,
         direnv=direnv,
     )
-    builder.generate(overwrite=force)
+    builder.generate(dest=output, overwrite=force)
 
-    if not click.confirm("Generate a .dockerignore?"):
-        return
-
-    generate_dockerignore(pkg_root, wf_type=workflow_type, overwrite=force)
+    generate_dockerignore(ignore_path, wf_type=workflow_type, overwrite=force)
 
 
 @main.command("generate-metadata")
@@ -364,8 +399,10 @@ def generate_metadata(
 
     if snakemake is True and nextflow is True:
         click.secho(
-            f"Please specify only one workflow type to generate metadata for. Use"
-            f" either `--snakemake` or `--nextflow`.",
+            (
+                "Please specify only one workflow type to generate metadata for. Use"
+                " either `--snakemake` or `--nextflow`."
+            ),
             fg="red",
         )
         raise click.exceptions.Exit(1)
@@ -379,12 +416,7 @@ def generate_metadata(
         if config_file is None:
             config_file = Path("nextflow_schema.json")
 
-        generate_metadata(
-            config_file,
-            metadata_root,
-            skip_confirmation=yes,
-            generate_defaults=not no_defaults,
-        )
+        generate_metadata(config_file, metadata_root, skip_confirmation=yes)
     else:
         from latch_cli.snakemake.config.parser import generate_metadata
 
@@ -506,9 +538,9 @@ def execute(
 ):
     """Drops the user into an interactive shell from within a task."""
 
-    from latch_cli.services.k8s.execute import exec
+    from latch_cli.services.k8s.execute import exec as _exec
 
-    exec(execution_id=execution_id, egn_id=egn_id, container_index=container_index)
+    _exec(execution_id=execution_id, egn_id=egn_id, container_index=container_index)
 
 
 @main.command("register")
@@ -615,6 +647,15 @@ def execute(
         "will be available to develop sessions."
     ),
 )
+@click.option(
+    "--dockerfile",
+    type=click.Path(exists=True, dir_okay=False, path_type=Path),
+    default=None,
+    help=(
+        "Path to a custom Dockerfile to use when registering. Default is to (1) use a Dockerfile in "
+        "the package root if one exists, or (2) generate one in .latch/Dockerfile if none exists."
+    ),
+)
 @requires_login
 def register(
     pkg_root: str,
@@ -631,6 +672,7 @@ def register(
     nf_execution_profile: Optional[str],
     mark_as_release: bool,
     staging: bool,
+    dockerfile: Optional[Path],
 ):
     """Register local workflow code to Latch.
 
@@ -653,6 +695,7 @@ def register(
             wf_module=workflow_module,
             progress_plain=(docker_progress == "auto" and not sys.stdout.isatty())
             or docker_progress == "plain",
+            dockerfile_path=dockerfile,
         )
 
         return
@@ -688,6 +731,7 @@ def register(
         use_new_centromere=use_new_centromere,
         cache_tasks=cache_tasks,
         mark_as_release=mark_as_release,
+        dockerfile_path=dockerfile,
     )
 
 
@@ -871,7 +915,7 @@ LDATA COMMANDS
 )
 @requires_login
 def cp(
-    src: List[str],
+    src: list[str],
     dest: str,
     progress: _Progress,
     verbose: bool,
@@ -879,8 +923,7 @@ def cp(
     cores: Optional[int] = None,
     chunk_size_mib: Optional[int] = None,
 ):
-    """
-    Copy files between Latch Data and local, or between two Latch Data locations.
+    """Copy files between Latch Data and local, or between two Latch Data locations.
 
     Behaves like `cp -R` in Unix. Directories are copied recursively. If any parents of dest do not exist, the copy will fail.
     """
@@ -933,10 +976,8 @@ def mv(src: str, dest: str, no_glob: bool):
 )
 @click.argument("paths", nargs=-1, shell_complete=remote_complete)
 @requires_login
-def ls(paths: Tuple[str], group_directories_first: bool):
-    """
-    List the contents of a Latch Data directory
-    """
+def ls(paths: tuple[str], group_directories_first: bool):
+    """List the contents of a Latch Data directory"""
 
     crash_handler.message = f"Unable to display contents of {paths}"
     crash_handler.pkg_root = str(Path.cwd())
@@ -1027,15 +1068,13 @@ def mkdir(remote_directory: str):
 @click.option("--cores", help="Number of cores to use for parallel syncing.", type=int)
 @requires_login
 def sync(
-    srcs: List[str],
+    srcs: list[str],
     dst: str,
     delete: bool,
     ignore_unsyncable: bool,
     cores: Optional[int] = None,
 ):
-    """
-    Update the contents of a remote directory with local data.
-    """
+    """Update the contents of a remote directory with local data."""
     from latch_cli.services.sync import sync
 
     # todo(maximsmol): remote -> local
@@ -1051,7 +1090,6 @@ NEXTFLOW COMMANDS
 @main.group()
 def nextflow():
     """Manage nextflow"""
-    pass
 
 
 @nextflow.command("version")
@@ -1059,8 +1097,7 @@ def nextflow():
 def version(pkg_root: Path):
     """Get the Latch version of Nextflow installed for the current project."""
 
-    with open(pkg_root / ".latch" / "nextflow_version", "r") as f:
-        version = f.read().strip()
+    version = (pkg_root / ".latch" / "nextflow_version").read_text().strip()
 
     click.secho(f"Nextflow version: {version}", fg="green")
 
@@ -1084,23 +1121,49 @@ def version(pkg_root: Path):
     default=None,
     help="Set execution profile for Nextflow workflow",
 )
+@click.option(
+    "--output",
+    "-o",
+    type=click.Path(path_type=Path),
+    default=None,
+    help=(
+        "Where to write the entrypoint file. Defaults to wf/custom_entrypoint.py. If the filename "
+        "does not end with a .py suffix, one will be appended."
+    ),
+)
+@click.option("--yes", "-y", is_flag=True, help="Skip the confirmation dialog.")
 def generate_entrypoint(
     pkg_root: Path,
     metadata_root: Optional[Path],
     nf_script: Path,
     execution_profile: Optional[str],
+    output: Optional[Path],
+    yes: bool,
 ):
     """Generate a `wf/entrypoint.py` file from a Nextflow workflow"""
 
-    import latch.types.metadata as metadata
+    from latch.types import metadata
     from latch_cli.nextflow.workflow import generate_nextflow_workflow
     from latch_cli.services.register.utils import import_module_by_path
 
-    dest = pkg_root / "wf" / "custom_entrypoint.py"
-    dest.parent.mkdir(exist_ok=True)
+    if output is None:
+        output = pkg_root / "wf" / "custom_entrypoint.py"
 
-    if dest.exists() and not click.confirm(
-        f"Nextflow entrypoint already exists at `{dest}`. Overwrite?"
+    output = output.with_suffix(".py")
+
+    if not yes and not click.confirm(
+        f"Will generate an entrypoint at {output}. Proceed?"
+    ):
+        raise click.exceptions.Abort
+
+    output.parent.mkdir(exist_ok=True)
+
+    if (
+        not yes
+        and output.exists()
+        and not click.confirm(
+            f"Nextflow entrypoint already exists at `{output}`. Overwrite?"
+        )
     ):
         return
 
@@ -1124,7 +1187,7 @@ def generate_entrypoint(
         raise click.exceptions.Exit(1)
 
     generate_nextflow_workflow(
-        pkg_root, metadata_root, nf_script, dest, execution_profile=execution_profile
+        pkg_root, metadata_root, nf_script, output, execution_profile=execution_profile
     )
 
 
@@ -1336,7 +1399,6 @@ POD COMMANDS
 @main.group()
 def pods():
     """Manage pods"""
-    pass
 
 
 @pods.command("stop")
@@ -1427,7 +1489,7 @@ def test_data_remove(object_url: str):
 def test_data_ls():
     """List test data objects."""
 
-    crash_handler.message = f"Unable to list objects within managed bucket"
+    crash_handler.message = "Unable to list objects within managed bucket"
     crash_handler.pkg_root = str(Path.cwd())
 
     from latch_cli.services.test_data.ls import ls
